<?php
class FileHandler
{
	private $_arr_json;
	private $_buffer;
	private $_section;
	private $_navDataText;
	private $_navQuickLinkText;
	private $_templateFilePath;
	private $_logger;
	private $_sort_array;
	private $_arr_root_node;
	
	public function __construct()
	{
		global $ne2_config_info;
		
		$this->_arr_json = array(
			'A' => array(),
			'Z' => array(),
			'S' => array()
		);
		$this->_buffer = '';
		$this->_section = 0;
		$this->_navDataText = '';
		$this->_navQuickLinkText = '';
//		$this->_templateFilePath = $_SERVER['DOCUMENT_ROOT'] . '/vkapp/NavEditor2/data/templates/seitenvorlage.html';
		$this->_templateFilePath = $ne2_config_info['app_path'] . 'data/templates/seitenvorlage.html';
		$this->_logger = null;
		$this->_sort_array = array(); // to sort and contain the partial parse result
		array_push($this->_sort_array, array('key' => 'A0')); // placeholder for U
		$this->_arr_root_node = array();
	}
	
	public function setTemplateFile($templPath)
	{
		$this->_templateFilePath = $templPath;
	}
	
	public function __destruct()
	{
		unset($this->arr_json);
	}
	
	public function setLogger($logger)
	{
		$this->_logger = $logger;
	}
	
	private function _nodeType($nodeKey)
	{
		$firstLetter = substr($nodeKey, 0, 1);
		return strtoupper($firstLetter);
	}
	
	private function _make_sort_key($ks) {
		$pad_len = 3; // max 999 item no. -_-
		$ksa = explode('-', $ks);
		for($i = 0; $i < count($ksa); $i++) {
			$t = $ksa[$i];
			$t = substr($t, 0, 1) . str_pad(substr($t, 1), $pad_len, '0', STR_PAD_LEFT);
			$ksa[$i] = $t;
		}
		return implode($ksa, '-');
	}
	
	function _extractKeyString($key_string)
	{
		$ret_arr = array(
			'key' => '',
			'info_text' => FALSE,
			'url' => FALSE,
			'sort_key' => ''
		);
		if(strtoupper($key_string) == 'U') {
			$ret_arr['key'] = 'U';
			return $ret_arr;
		}
		$test = substr($key_string, strlen($key_string) - 1);
		if(is_numeric($test)) {
			$ret_arr['key'] = $key_string;
			$ret_arr['sort_key'] = $this->_make_sort_key($key_string);
			return $ret_arr;
		} else {
			$ret_arr['key'] = substr($key_string, 0, strrpos($key_string, '-'));
			$sp = explode('-', $key_string);
			if(strtoupper($sp[count($sp) - 1]) == 'URL') {
				$ret_arr['url'] = TRUE;
			} elseif(strtoupper($sp[count($sp) - 1]) == 'INFOTEXT') {
				$ret_arr['info_text'] = TRUE;
			}
			return $ret_arr;
		}
	}
	
	private function _updateSortArray($key, $vk, $vv)
	{
		foreach($this->_sort_array as &$sa) {
			if($sa['key'] == $key) {
				$sa[$vk] = $vv;
				break;
			}
		}
		
		if($key == 'U') {
			$this->_arr_root_node[$vk] = $vv;
		}
	}
	
	private function _filterChars($source)
	{
		$filter = array('/', '\\');
		return str_replace($filter, '', trim($source));
	}
	
	private function _parseLine($line_string)
	{
		$arr_buf = explode("\t", $line_string);
		if(count($arr_buf) <= 1) {
			return FALSE;
		}
		
		$arr_line = array();
		$key_string = $this->_filterChars($arr_buf[0]); // filter the bad chars
		$key_ext = $this->_extractKeyString($key_string);
		if(($key_ext['url'] == FALSE) && ($key_ext['info_text'] == FALSE)) {
			$arr_line['sort_key'] = $key_ext['sort_key']; // Key for sorting
			$arr_line['key'] = $key_ext['key']; // Key
			$arr_line['title'] = html_entity_decode($arr_buf[1], ENT_COMPAT, 'UTF-8'); // Title
			
			$arr_line['child'] = array(); // Hold for child-folders
			
			$arr_line['alias'] = isset($arr_buf[2]) ? $this->_filterChars($arr_buf[2]) : '';
			$arr_line['accesskey'] = isset($arr_buf[3]) ? $arr_buf[3] : '';
			$arr_line['email'] = isset($arr_buf[4]) ? $arr_buf[4] : '';
			
			$arr_line['path'] = '';
			$arr_line['quicklink'] = false;
			$arr_line['displaylink'] = false;
			
			$arr_line['url'] = '';
			$arr_line['info_text'] = '';
			
			if($key_ext['key'] != 'U') {
				array_push($this->_sort_array, $arr_line);
			} else {
				$this->_arr_root_node = $arr_line;
			}
		} else {
			if($key_ext['url'] == TRUE) {
				$mkey = $key_ext['key'];
				$vkey = 'url';
				$vval = $arr_buf[1];
				$this->_updateSortArray($mkey, $vkey, $vval);
			}
			
			if($key_ext['info_text'] == TRUE) {
				$mkey = $key_ext['key'];
				$vkey = 'info_text';
				$vval = $arr_buf[1];
				$this->_updateSortArray($mkey, $vkey, $vval);
			}
		}
		
		return TRUE;
	}
	
	function _prepareTree()
	{
		// !!!
		sort($this->_sort_array);
		
		$this->_sort_array[0] = $this->_arr_root_node;
		
		foreach($this->_sort_array as $sa) {
			$this->_addToJSONArray($sa);
		}
	}
	
	private function _addToJSONArray($arr_part)
	{
		$check_key = $arr_part['key'];
		$arr_check_key = explode("-", $check_key);
		$nodeType = $this->_nodeType($check_key);
		
		switch(count($arr_check_key)) {
			case 1: // A1
				switch($nodeType) {
					case 'U':
					case 'A':
						array_push($this->_arr_json['A'], $arr_part);
						break;
					case 'Z':
						array_push($this->_arr_json['Z'], $arr_part);
						break;
					case 'S':
						array_push($this->_arr_json['S'], $arr_part);
						break;
					default:
						return FALSE;
						break;
				}
				break;
			default:
				$target = &$this->_arr_json['A'];
				for($i = 1; $i < count($arr_check_key); $i++) { // start from 2nd elem.
					$target = &$target[count($target) - 1]['child'];
				}
				array_push($target, $arr_part);
				break;
		}
	}
	
	private function _setQuickLink($key, &$nodes)
	{
		for($i = 0; $i < count($nodes); $i++) {
			if($nodes[$i]['key'] == $key) {
				$nodes[$i]['quicklink'] = true;
				return;
			}
			if(count($nodes[$i]['child']) > 0) {
				$this->_setQuickLink($key, $nodes[$i]['child']);
			}
		}
	}
	
	public function loadJSONFromFile($file_path)
	{
		$fh = fopen($file_path, 'r') or die("Cannot open the file! $php_errormsg");
		while(!feof($fh)) {
			$this->_buffer = fgets($fh);
			$this->_buffer = trim($this->_buffer);
			if((strlen($this->_buffer) == 0) || (substr($this->_buffer, 0, 1) == '#')) {
				continue;
			}
			
			switch($this->_buffer) {
				case '<navigation>':
					$this->_section = 1;
					break;
				case '<quicklinks>':
					$this->_section = 2;
					break;
				case '<accesskeys>':
					$this->_section = 3;
					break;
				case '<displaylinks>':
					$this->_section = 4;
					break;
				case '</navigation>':
				case '</quicklinks>':
				case '</accesskeys>':
				case '</displaylinks>':
					$this->_section = 0;
					break;
				default:
					if($this->_section == 1) {
						if(!$this->_parseLine($this->_buffer)) {
							continue;
						}
					}
					if($this->_section == 2) {
						$this->_setQuickLink($this->_buffer, $this->_arr_json['A']);
					}
					break;
			}
		}
		fclose($fh);
		
		$this->_prepareTree();
		
		return $this->_arr_json;
	}
	
	public function setJSONArray($jsonArray)
	{
		$this->_arr_json = $jsonArray;
	}
	
	private function _doLog($logText)
	{
		if(!is_null($this->_logger)) {
			$this->_logger->Log($logText);
		}
	}
	
	private function _file_exists_case($strUrl) {
		$realPath = str_replace('\\', '/', realpath($strUrl));
		if(file_exists($strUrl) && $realPath == $strUrl) {
			return 1; // File exists, with correct case
		} elseif(file_exists($realPath)) {
			return 2; // File exists, but wrong case
		} else {
			return 0; // File does not exist
		}
	}
	
	public function checkPath($path)
	{
		$path = $_SERVER['DOCUMENT_ROOT'] . $path;
		$dir = substr($path, 0, strrpos($path, "/"));
		if(!file_exists($path)) {
			if(!is_dir($dir)) {
				$this->_doLog('mkdir: ' . $dir);
				umask(0022); // !!!
				mkdir($dir, 0777, TRUE);
			}
			$this->_doLog('copy: ' . $this->_templateFilePath . ' -> ' . $path);
			copy($this->_templateFilePath, $path);
			chmod($path, 0777);
		}
	}
	
	private function _buildNavDataText($arrPart)
	{
		static $indent = 0;
		for($i = 0; $i < count($arrPart); $i++) {
			$indent++;
			$ftabs = str_repeat("\t", $indent);
			$this->_navDataText .= $ftabs . $arrPart[$i]['key'];
			$this->_navDataText .= "\t" . htmlentities($arrPart[$i]['title'], ENT_COMPAT, 'UTF-8');
			$this->_navDataText .= "\t" . $arrPart[$i]['alias'];
			$this->_navDataText .= "\t" . $arrPart[$i]['accesskey'];
			$this->_navDataText .= "\t" . $arrPart[$i]['email'];
			if(strlen($arrPart[$i]['url']) > 0) {
				$this->_navDataText .= "\n" . $ftabs . $arrPart[$i]['key'] . "-URL\t" . $arrPart[$i]['url'];
			}
			if(strlen($arrPart[$i]['info_text']) > 0) {
				$this->_navDataText .= "\n" . $ftabs . $arrPart[$i]['key'] . "-Infotext\t" . $arrPart[$i]['info_text'];
			}
			$this->_navDataText .= "\n";
			
			$this->_navQuickLinkText .= $arrPart[$i]['quicklink'] == true ? $arrPart[$i]['key'] . "\n" : '';
			
			if(strlen($arrPart[$i]['url']) == 0) {
				$this->checkPath($arrPart[$i]['path']);
			}
			
			if(count($arrPart[$i]['child']) > 0) {
				$this->_buildNavDataText($arrPart[$i]['child']);
			}
			$indent--;
		}
	}
	
	public function saveJSONToFile($filePath)
	{
		$this->_buildNavDataText($this->_arr_json['A']);
		$this->_buildNavDataText($this->_arr_json['Z']);
		$this->_buildNavDataText($this->_arr_json['S']);
		
		// backup
		$backup_str = '_bak_' . date('YmdHis') . '.txt';
		$new_path = $filePath . $backup_str;
		if(!copy($filePath, $new_path)) {
			throw new Exception('Creating backup error!');
		}
		
		$old_content = file_get_contents($filePath);
		$new_content = preg_replace(array('/<navigation>((\n|\r\n|.)*?)<\/navigation>/', '/<quicklinks>((\n|\r\n|.)*?)<\/quicklinks>/'), array('[[[navigation]]]', '[[[quicklinks]]]'), $old_content);
		
		$new_content1 = str_replace(array('[[[navigation]]]', '[[[quicklinks]]]'), array("<navigation>\n" . $this->_navDataText . "</navigation>\n", "<quicklinks>\nU\n" . $this->_navQuickLinkText . "</quicklinks>\n"), $new_content);
		
		$fh = fopen($filePath, 'w+') or die("Cannot open the file! $php_errormsg");
//		fwrite($fh, "<navigation>\n" . $this->_navDataText . "</navigation>\n");
//		fwrite($fh, "<accesskeys>\n\tU\t0\tstart\n</accesskeys>\n");
//		fwrite($fh, "<quicklinks>\nU\n" . $this->_navQuickLinkText . "</quicklinks>\n");
//		fwrite($fh, "<displaylinks>\n</displaylinks>\n");
		fwrite($fh, $new_content1);
		fclose($fh);
	}
	
	public function UpdateExistedPageLogos($logo_content_json, $dir)
	{
		if(is_dir($dir)) {
			if($dh = opendir($dir)) {
				while(FALSE !== ($file = readdir($dh))) {
					// escaped dirs
					if($file != '.' && $file != '..' && $file != 'css' && $file != 'grafiken' && $file != 'img' && $file != 'Smarty' && $file != 'ssi' && $file != 'univis' && $file != 'vkapp' && $file != 'vkdaten' && $file != 'xampp') {
						if(is_dir($dir . $file . '/')) {
							// recursively
							$this->UpdateExistedPageLogos($logo_content_json, $dir . $file . '/');
						} else {
							if(strpos($file, '.shtml') !== FALSE) {
								// begin update logo html
//								echo($dir . $file . '<br />');
								$this->doReplaceLogo($logo_content_json, $dir . $file);
							}
						}
					}
				}
				closedir($dh);
			}
		}
	}
	
	public function UpdateStartPageLogo()
	{
		// just remove the link
		$start_page_path = $_SERVER['DOCUMENT_ROOT'] . '/index.shtml';
		if(file_exists($start_page_path)) {
			$fcontent = file_get_contents($start_page_path);
			$pat = '/<div id="logo">((\n|.)*?)<\/div>/i';
			preg_match($pat, $fcontent, $matches);
			$rcontent = $matches[1];
			$rcontent = str_replace(array('<a href="/">', '</a>'), '', $rcontent);
			$newContent = preg_replace($pat, '<div id="logo">' . $rcontent . '</div>', $fcontent);
			file_put_contents($start_page_path, $newContent);
		}
	}
	
	private function doReplaceLogo($logo_content_json, $page_file_path)
	{
		$fcontent = file_get_contents($page_file_path);
		$data = json_decode(stripslashes($logo_content_json));
		$pat = '/<div id="logo">(\n|.)*?<\/div>/i';
		$rcontent = '';
		if(strlen($data->content_img) < 1) {
			$rcontent = '<p><a href="/">' . htmlentities($data->content_text, ENT_COMPAT, 'UTF-8') . '</a></p>';
		} else {
			$rcontent = '<a href="/">' . $data->content_img . '</a>';
		}
		$newContent = preg_replace($pat, '<div id="logo">' . $rcontent . '</div>', $fcontent);
		file_put_contents($page_file_path, $newContent);
		
		$this->replaceTitleTagContent($data->content_text, $page_file_path);
	}
	
	private function replaceTitleTagContent($new_title_text, $page_file_path)
	{
		$fcontent = file_get_contents($page_file_path);
		
		// <h1>...</h1> / get page-title
		$pattern2 = '/(<div id="titel">)((\n|.)*?)(<\/div>)/i';
		preg_match($pattern2, $fcontent, $matches);
		$page_title = $matches[2];
		$page_title = str_replace(array('<h1>', '</h1>'), '', $page_title);
		// <title>...</title>
		$pattern1 = '/<title>((.)*?)<\/title>/i';
		$fcontent = preg_replace($pattern1, '<title>' . htmlentities($new_title_text, ENT_COMPAT, 'UTF-8') . ': ' . $page_title . '</title>', $fcontent);
		
		file_put_contents($page_file_path, $fcontent);
	}
	
	public function UpdateTitleFile($titile_text)
	{
		global $ne2_config_info;
		
		$titleFilePath = $ne2_config_info['app_path'] . 'data/' . $ne2_config_info['current_site_title_file'];
		if(file_exists($titleFilePath)) {
			file_put_contents($titleFilePath, htmlentities($titile_text, ENT_COMPAT, 'UTF-8'));
		}
	}
}
?>
